@@ -90,7 +90,7 @@ do
   ch1=`replace_strange_char $i`
   echo "	this.ENGINE_${ch1} = ${CLASSNAME}.ENGINE_${ch1};"
 done
-[ -n "$LEXER" ] && echo "	this._unreadl = null;"
+echo "	this._unreadl = null;"
 echo '}'
 echo
 echo "function ${CLASSNAME}_TokenException() {"
@@ -159,7 +159,34 @@ EOF
 
 cat field
 if [ -z "$LEXER" ]; then
-  _putreadf _read1
+  _putreadf _read1l
+  cat << EOF
+	
+${CLASSNAME}.prototype._read1 = function(stream) {
+	var c;
+
+	if(this._unreadl != null) {
+		c = this._unreadl;
+		this._unreadl = null;
+EOF
+  if [ -z "$TYPE" -a -z "$NEWLINEMODE" ]; then
+    cat << EOF
+	} else if((c = this._read1l(stream)) == 13 && (c = this._read1l(stream)) != 10) {
+		this._unreadl = c;
+		c = 13;
+	}
+EOF
+  else
+    cat << EOF
+	} else {
+		c = this._read1l(stream);
+	}
+EOF
+  fi
+  cat << EOF
+	return c;
+}
+EOF
 else
   _putreadf _read1l
   nina_template.nfa.js.sub2.sh "$LEXER" "$LEXER" "$CLASSNAME" "$LCTYPE"
@@ -168,12 +195,25 @@ else
 ${CLASSNAME}.prototype._read1ul = function(stream) {
 	var c;
 
-	if(this._unreadl == null) {
-		c = this._read1l(stream);
-	} else {
+	if(this._unreadl != null) {
 		c = this._unreadl;
 		this._unreadl = null;
+EOF
+  if [ -z "$LTYPE" -a -z "$NEWLINEMODE" ]; then
+    cat << EOF
+	} else if((c = this._read1l(stream)) == 13 && (c = this._read1l(stream)) != 10) {
+		this._unreadl = c;
+		c = 13;
 	}
+EOF
+  else
+    cat << EOF
+	} else {
+		c = this._read1l(stream);
+	}
+EOF
+  fi
+  cat << EOF
 	return c;
 }
 
@@ -327,9 +367,17 @@ ${CLASSNAME}.prototype._f_LOOKAHEAD = function(c) {
 ${CLASSNAME}.prototype.__copy_lookahead = function(p) {
 	var a, k;
 
+	if(this.__lookahead_ptr > 0) {
+		a = [];
+		for(k = 0; k < a.length; k++) {
+			a[k] = this.__lookaheadw[k + this.__lookahead_ptr];
+		}
+		__lookahead = a;
+	}
+
 	if(this.__lookahead == null) {
 		a = [];
-	} else if(this.__lookaheadw_ptr < this.__lookahead.length) {
+	} else if(this.__lookaheadw_ptr < this.__lookahead.length - this.__lookahead_ptr) {
 		a = this.__lookahead;
 	} else {
 		a = [];
@@ -672,13 +720,13 @@ ${CLASSNAME}.prototype._parse = function(rd, x, rt, st) {
 					st[0] = NINA_YIELD;  return null;
 				}
 EOF
-  [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
+[ -n "$ENABLE_BACKTRACK" ] && cat << EOF
 				if(this.__backtrack_flg) {
 					this.__write_backtrack(c);
 					this.__backtrack_flg = false;
 				}
 EOF
-  cat << EOF
+cat << EOF
 			} else if(a < 0) {
 				this.__logprint("entering " + this.__stk[this.__slen - 1].toString());
 				return c;
@@ -687,18 +735,18 @@ EOF
 				this._f_UNGET(c);
 				st[0] = NINA_ACCEPT;  return null;
 EOF
-  [ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
+[ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
 			} else if(this.__lookaheadw_ptr >= 0) {
 				this.__logprint("match failed: try lookahead: ", c);
 				this._f_LOOKAHEAD(c);
 				this._f_LOOKAHEAD_RB();
 				b = en.accepted(this);
 EOF
-  [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
+[ -n "$ENABLE_BACKTRACK" ] && cat << EOF
 			} else if(this._f_BACKTRACK(c)) {
 				this.__logprint("match failed: try backtracking: ", c);
 EOF
-  cat << EOF
+cat << EOF
 			} else if(c == null) {
 				if(!b)  throw "${TOKENERROR}";
 				st[0] = NINA_ACCEPT;  return null;
@@ -717,17 +765,17 @@ EOF
 				this.__logprint("accept " + this.__stk[this.__slen - 1].toString());
 				st[0] = NINA_ACCEPT;  return null;
 EOF
-  [ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
+[ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
 			} else if(this.__lookaheadw_ptr >= 0) {
 				this.__logprint("match failed: try lookahead: ", c);
 				this._f_LOOKAHEAD_RB();
 				b = en.accepted(this);
 EOF
-  [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
+[ -n "$ENABLE_BACKTRACK" ] && cat << EOF
 			} else if(this._f_BACKTRACK(c)) {
 				this.__logprint("match failed: try backtracking: ", c);
 EOF
-  cat << EOF
+cat << EOF
 			} else if(c == null) {
 				if(!b)  throw "${TOKENERROR}";
 				st[0] = NINA_ACCEPT;  return null;
@@ -882,6 +930,10 @@ ${CLASSNAME}.parseStdin = function() {
 	return ${CLASSNAME}.parseStream(
 			new java.io.InputStreamReader(java.lang.System['in']));
 }
+
+function puts(s) {
+	CONSOLE.println(s);
+}
 EOF
 
 for i in $SUBAUTOMATA
